				-- Top Level Structural Model, TOP_SPIM
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;

ENTITY TOP_SPIM IS

	PORT( reset, clock					: IN 	STD_LOGIC; 
		-- Output important signals to pins for easy display in Simulator
		PC, ALU_result_out, read_data_1_out, 
		read_data_2_out, write_data_out	: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
     	Instruction_out					: OUT 	STD_LOGIC_VECTOR( 31 DOWNTO 0 );
		Branch_out, Zero_out, Memwrite_out, 
		Regwrite_out					: OUT 	STD_LOGIC );
END 	TOP_SPIM;

ARCHITECTURE structure OF TOP_SPIM IS

	COMPONENT Ifetch
   	     PORT(	Instruction			: OUT 	STD_LOGIC_VECTOR( 31 DOWNTO 0 );
        		PC_plus_4_out 		: OUT  	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		Add_result 			: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		Branch 				: IN 	STD_LOGIC;
        		Zero 				: IN 	STD_LOGIC;
        		PC_out 				: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		clock,reset 		: IN 	STD_LOGIC );
	END COMPONENT;

	COMPONENT IFID_REG
        PORT(   DIN_Instruction          : IN    STD_LOGIC_VECTOR( 31 DOWNTO 0 );
                DIN_PC_plus_4_out        : IN    STD_LOGIC_VECTOR( 7 DOWNTO 0 );
                DOUT_Instruction         : OUT   STD_LOGIC_VECTOR( 31 DOWNTO 0 );
                DOUT_PC_plus_4_out       : OUT   STD_LOGIC_VECTOR( 7 DOWNTO 0 );
                clock, reset             : IN    STD_LOGIC);
	END COMPONENT;

	COMPONENT Idecode
 	     PORT(	read_data_1 		: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		read_data_2 		: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		Instruction 		: IN 	STD_LOGIC_VECTOR( 31 DOWNTO 0 );
        		read_data 			: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		ALU_result 			: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		RegWrite, MemtoReg 	: IN 	STD_LOGIC;
        		RegDst 				: IN 	STD_LOGIC;
        		Sign_extend 		: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				write_register_address_out	: OUT	STD_LOGIC_VECTOR( 4 DOWNTO 0 );
				write_register_address		: IN	STD_LOGIC_VECTOR( 4 DOWNTO 0 );
        		clock, reset		: IN 	STD_LOGIC );
	END COMPONENT;

	COMPONENT control
	     PORT( 	Opcode 				: IN 	STD_LOGIC_VECTOR( 5 DOWNTO 0 );
             	RegDst 				: OUT 	STD_LOGIC;
             	ALUSrc 				: OUT 	STD_LOGIC;
             	MemtoReg 			: OUT 	STD_LOGIC;
             	RegWrite 			: OUT 	STD_LOGIC;
             	MemRead 			: OUT 	STD_LOGIC;
             	MemWrite 			: OUT 	STD_LOGIC;
             	Branch 				: OUT 	STD_LOGIC;
             	ALUop 				: OUT 	STD_LOGIC_VECTOR( 1 DOWNTO 0 );
             	clock, reset		: IN 	STD_LOGIC );
	END COMPONENT;

	COMPONENT IDEX_REG
        PORT(   DDIN_Instruction          : IN    STD_LOGIC_VECTOR( 31 DOWNTO 0 );
                DDIN_PC_plus_4_out        : IN    STD_LOGIC_VECTOR( 7 DOWNTO 0 );
                DIN_read_data_1			  : IN    STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DIN_read_data_2           : IN    STD_LOGIC_VECTOR( 7 DOWNTO 0 );			
				DDOUT_Instruction         : OUT   STD_LOGIC_VECTOR( 31 DOWNTO 0 );
                DDOUT_PC_plus_4_out       : OUT   STD_LOGIC_VECTOR( 7 DOWNTO 0 );
                DOUT_read_data_1		  : OUT   STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DOUT_read_data_2          : OUT   STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DIN_ALUop 				  : IN    STD_LOGIC_VECTOR( 1 DOWNTO 0 );
				DIN_RegDst 			  	  : IN 	  STD_LOGIC;
				DIN_ALUSrc 			      : IN 	  STD_LOGIC;
				DIN_MemtoReg 			  : IN 	  STD_LOGIC;
				DIN_RegWrite			  : IN 	  STD_LOGIC;
				DIN_MemRead 			  : IN 	  STD_LOGIC;
				DIN_MemWrite 			  : IN 	  STD_LOGIC;
				DIN_Branch 			  	  : IN 	  STD_LOGIC;
				DOUT_ALUop 				  : OUT   STD_LOGIC_VECTOR( 1 DOWNTO 0 );
				DOUT_RegDst 			  : OUT   STD_LOGIC;
				DOUT_ALUSrc 			  : OUT   STD_LOGIC;
				DOUT_MemtoReg 			  : OUT   STD_LOGIC;
				DOUT_RegWrite			  : OUT   STD_LOGIC;
				DOUT_MemRead 			  : OUT   STD_LOGIC;
				DOUT_MemWrite 			  : OUT   STD_LOGIC;
				DOUT_Branch 			  : OUT   STD_LOGIC;
				DIN_Sign_extend			  : IN	  STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DOUT_Sign_extend		  : OUT	  STD_LOGIC_VECTOR( 7 DOWNTO 0 );				
				DIN_write_register_address		: IN	STD_LOGIC_VECTOR( 4 DOWNTO 0 );
				DOUT_write_register_address		: OUT	STD_LOGIC_VECTOR( 4 DOWNTO 0 );
				clock, reset              : IN    STD_LOGIC);
	END COMPONENT;

	COMPONENT  Execute
   	     PORT(	Read_data_1 		: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
                Read_data_2 		: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
               	Sign_Extend 		: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
               	Function_opcode		: IN 	STD_LOGIC_VECTOR( 5 DOWNTO 0 );
               	ALUOp 				: IN 	STD_LOGIC_VECTOR( 1 DOWNTO 0 );
               	ALUSrc 				: IN 	STD_LOGIC;
               	Zero 				: OUT	STD_LOGIC;
               	ALU_Result 			: OUT	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
               	Add_Result 			: OUT	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
               	PC_plus_4 			: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
               	clock, reset		: IN 	STD_LOGIC );
	END COMPONENT;

	COMPONENT EXMEM_REG
        PORT(   DIN_Add_result 		: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DIN_ALU_result 		: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DDIN_read_data_2    : IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DOUT_Add_result		: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DOUT_ALU_result		: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DDOUT_read_data_2   : OUT   STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DDIN_MemtoReg 		: IN 	STD_LOGIC;
				DDIN_RegWrite		: IN 	STD_LOGIC;
				DDIN_MemRead 		: IN 	STD_LOGIC;
				DDIN_MemWrite 		: IN 	STD_LOGIC;
				DDIN_Branch 		: IN 	STD_LOGIC;
				DDOUT_MemtoReg 		: OUT   STD_LOGIC;
				DDOUT_RegWrite		: OUT   STD_LOGIC;
				DDOUT_MemRead 		: OUT   STD_LOGIC;
				DDOUT_MemWrite 		: OUT   STD_LOGIC;
				DDOUT_Branch 		: OUT   STD_LOGIC;
				DIN_Zero			: IN	STD_LOGIC;
				DOUT_Zero			: OUT 	STD_LOGIC;
				DDIN_write_register_address		: IN	STD_LOGIC_VECTOR( 4 DOWNTO 0 );
				DDOUT_write_register_address	: OUT	STD_LOGIC_VECTOR( 4 DOWNTO 0 );
                clock, reset        : IN    STD_LOGIC);
	END COMPONENT;

	COMPONENT dmemory
	     PORT(	read_data 			: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		address 			: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		write_data 			: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
        		MemRead, Memwrite 	: IN 	STD_LOGIC;
        		Clock,reset			: IN 	STD_LOGIC );
	END COMPONENT;

	COMPONENT MEMWB_REG
		PORT(	DIN_read_data		: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DDIN_ALU_result		: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DOUT_read_data		: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DDOUT_ALU_result	: OUT 	STD_LOGIC_VECTOR( 7 DOWNTO 0 );
				DDDIN_MemtoReg 		: IN 	STD_LOGIC;
				DDDIN_RegWrite		: IN 	STD_LOGIC;
				DDDOUT_MemtoReg 	: OUT   STD_LOGIC;
				DDDOUT_RegWrite		: OUT   STD_LOGIC;
				DDDIN_write_register_address	: IN	STD_LOGIC_VECTOR( 4 DOWNTO 0 );
				DDDOUT_write_register_address	: OUT	STD_LOGIC_VECTOR( 4 DOWNTO 0 );				 
                clock, reset        : IN 	STD_LOGIC);
	END COMPONENT;

					-- declare signals used to connect VHDL components
	SIGNAL PC_plus_4 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL read_data_1 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL read_data_2 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL Sign_Extend 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL Add_result 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL ALU_result 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL read_data 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL ALUSrc 			: STD_LOGIC;
	SIGNAL Branch 			: STD_LOGIC;
	SIGNAL RegDst 			: STD_LOGIC;
	SIGNAL Regwrite 		: STD_LOGIC;
	SIGNAL Zero 			: STD_LOGIC;
	SIGNAL MemWrite 		: STD_LOGIC;
	SIGNAL MemtoReg 		: STD_LOGIC;
	SIGNAL MemRead 			: STD_LOGIC;
	SIGNAL ALUop 			: STD_LOGIC_VECTOR(  1 DOWNTO 0 );
	SIGNAL Instruction		: STD_LOGIC_VECTOR( 31 DOWNTO 0 );
	SIGNAL write_register_address	: STD_LOGIC_VECTOR(  4 DOWNTO 0 );

			-- signals used to connect components to pipeline registers
			-- IF/ID Reg signals 
	SIGNAL D_Instruction    : STD_LOGIC_VECTOR( 31 DOWNTO 0 );
	SIGNAL D_PC_plus_4 		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );

			--ID/EX Reg signals
	SIGNAL DD_Instruction   : STD_LOGIC_VECTOR( 31 DOWNTO 0 );
	SIGNAL DD_PC_plus_4 	: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL D_read_data_1 	: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL D_read_data_2 	: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL D_ALUop 			: STD_LOGIC_VECTOR( 1 DOWNTO 0 );
	SIGNAL D_RegDst 		: STD_LOGIC;
	SIGNAL D_ALUSrc 		: STD_LOGIC;
	SIGNAL D_MemtoReg 		: STD_LOGIC;
	SIGNAL D_RegWrite		: STD_LOGIC;
	SIGNAL D_MemRead 		: STD_LOGIC;
	SIGNAL D_MemWrite 		: STD_LOGIC;
	SIGNAL D_Branch 		: STD_LOGIC;
	SIGNAL D_Sign_extend	: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL D_write_register_address		: STD_LOGIC_VECTOR( 4 DOWNTO 0 );


			--EX/MEM Reg signals
	SIGNAL D_Add_result		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL D_ALU_result		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL DD_read_data_2	: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL DD_MemtoReg 		: STD_LOGIC;
	SIGNAL DD_RegWrite		: STD_LOGIC;
	SIGNAL DD_MemRead 		: STD_LOGIC;
	SIGNAL DD_MemWrite 		: STD_LOGIC;
	SIGNAL DD_Branch 		: STD_LOGIC;
	SIGNAL D_ZERO			: STD_LOGIC;
	SIGNAL DD_write_register_address	: STD_LOGIC_VECTOR( 4 DOWNTO 0 );

			--MEM/WB Reg signals
	SIGNAL D_read_data		: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL DD_ALU_result	: STD_LOGIC_VECTOR( 7 DOWNTO 0 );
	SIGNAL DDD_MemtoReg 	: STD_LOGIC;
	SIGNAL DDD_RegWrite		: STD_LOGIC;
	SIGNAL DDD_write_register_address	: STD_LOGIC_VECTOR( 4 DOWNTO 0 );


BEGIN
					-- copy important signals to output pins for easy 
					-- display in Simulator
   Instruction_out 	<= Instruction;
   ALU_result_out 	<= ALU_result;
   read_data_1_out 	<= read_data_1;
   read_data_2_out 	<= read_data_2;
   write_data_out  	<= read_data WHEN MemtoReg = '1' ELSE ALU_result;
   Branch_out 		<= Branch;
   Zero_out 		<= Zero;
   RegWrite_out 	<= RegWrite;
   MemWrite_out 	<= MemWrite;

					-- connect the MIPS components   
  IFE : Ifetch
	PORT MAP (	Instruction 	=> Instruction,
    	    	PC_plus_4_out 	=> PC_plus_4,
				Add_result 		=> D_Add_result,
				Branch 			=> DD_Branch,
				Zero 			=> D_Zero,
				PC_out 			=> PC,        		
				clock 			=> clock,  
				reset 			=> reset );

  	IFID : IFID_REG
		PORT MAP ( 	DIN_Instruction		=> Instruction,
                	DIN_PC_plus_4_out	=> PC_plus_4,
                	DOUT_Instruction    => D_Instruction,
                	DOUT_PC_plus_4_out  => D_PC_plus_4,
                	clock				=> clock,
					reset				=> reset );					

   ID : Idecode
   	PORT MAP (	read_data_1 	=> read_data_1,
        		read_data_2 	=> read_data_2,
        		Instruction 	=> D_Instruction,
        		read_data 		=> D_read_data,
				ALU_result 		=> DD_ALU_result,
				RegWrite 		=> DDD_RegWrite,
				MemtoReg 		=> DDD_MemtoReg,
				RegDst 			=> RegDst,
				Sign_extend 	=> Sign_extend,
				write_register_address_out	=> write_register_address,
				write_register_address		=> DDD_write_register_address,
        		clock 			=> clock,  
				reset 			=> reset );

   CTL:   control
	PORT MAP ( 	Opcode 			=> D_Instruction( 31 DOWNTO 26 ),
				RegDst 			=> RegDst,
				ALUSrc 			=> ALUSrc,
				MemtoReg 		=> MemtoReg,
				RegWrite 		=> RegWrite,
				MemRead 		=> MemRead,
				MemWrite 		=> MemWrite,
				Branch 			=> Branch,
				ALUop 			=> ALUop,
                clock 			=> clock,
				reset 			=> reset );

	IDEX : IDEX_REG
		PORT MAP (	DDIN_Instruction		=> D_Instruction,          
                    DDIN_PC_plus_4_out		=> D_PC_plus_4,
                    DIN_read_data_1			=> read_data_1,
				    DIN_read_data_2			=> read_data_2,
				    DDOUT_Instruction       => DD_Instruction,  
                    DDOUT_PC_plus_4_out     => DD_PC_plus_4,
                    DOUT_read_data_1		=> D_read_data_1,
				    DOUT_read_data_2		=> D_read_data_2,
					DIN_ALUop 				=> ALUop,
					DIN_RegDst				=> RegDst,
					DIN_ALUSrc				=> ALUsrc,
					DIN_MemtoReg 			=> MemtoReg,
					DIN_RegWrite			=> RegWrite,
					DIN_MemRead 			=> MemRead,
					DIN_MemWrite 			=> MemWrite,
					DIN_Branch 			  	=> Branch,
					DOUT_ALUop 				=> D_ALUop,
					DOUT_RegDst 			=> D_RegDst,
					DOUT_ALUSrc 			=> D_ALUSrc,
					DOUT_MemtoReg			=> D_MemtoReg,
					DOUT_RegWrite			=> D_RegWrite,
					DOUT_MemRead			=> D_MemRead,
					DOUT_MemWrite			=> D_MemWrite,
					DOUT_Branch				=> D_Branch,
					DIN_Sign_extend			=> Sign_extend,
					DOUT_Sign_extend		=> D_Sign_extend,
					DIN_write_register_address 	=> write_register_address,
					DOUT_write_register_address => D_write_register_address,
                    clock					=> clock,
					reset					=> reset );

   EXE:  Execute
   	PORT MAP (	Read_data_1 	=> D_read_data_1,
             	Read_data_2 	=> D_read_data_2,
				Sign_extend 	=> D_Sign_extend,
                Function_opcode	=> DD_Instruction( 5 DOWNTO 0 ),
				ALUOp 			=> D_ALUop,
				ALUSrc 			=> D_ALUSrc,
				Zero 			=> Zero,
                ALU_Result		=> ALU_Result,
				Add_Result 		=> Add_Result,
				PC_plus_4		=> DD_PC_plus_4,
                Clock			=> clock,
				Reset			=> reset );

	EXMEM : EXMEM_REG
		PORT MAP ( 	DIN_Add_result		=> Add_result,
					DIN_ALU_result		=> ALU_result,
					DDIN_read_data_2	=> D_read_data_2,
					DOUT_Add_result		=> D_Add_result,
					DOUT_ALU_result		=> D_ALU_result,
					DDOUT_read_data_2 	=> DD_read_data_2,
					DDIN_MemtoReg 		=> D_MemtoReg,
					DDIN_RegWrite		=> D_RegWrite,
					DDIN_MemRead 		=> D_MemRead,
					DDIN_MemWrite 		=> D_MemWrite,
					DDIN_Branch 		=> D_Branch,
					DDOUT_MemtoReg		=> DD_MemtoReg,
					DDOUT_RegWrite		=> DD_RegWrite,
					DDOUT_MemRead		=> DD_MemRead,
					DDOUT_MemWrite		=> DD_MemWrite,
					DDOUT_Branch		=> DD_Branch,
					DIN_Zero			=> Zero,
					DOUT_Zero			=> D_Zero,
					DDIN_write_register_address		=> D_write_register_address,
					DDOUT_write_register_address	=> DD_write_register_address,
                	clock				=> clock, 
					reset				=> reset );

   MEM:  dmemory
	PORT MAP (	read_data 		=> read_data,
				address 		=> D_ALU_Result,
				write_data 		=> DD_read_data_2,
				MemRead 		=> DD_MemRead, 
				Memwrite 		=> DD_MemWrite, 
                clock 			=> clock,  
				reset 			=> reset );

	MEMWB : MEMWB_REG
		PORT MAP (	DIN_read_data		=> read_data,
					DDIN_ALU_result		=> D_ALU_Result,
					DOUT_read_data		=> D_read_data,
					DDOUT_ALU_result	=> DD_ALU_Result,
					DDDIN_MemtoReg 		=> DD_MemtoReg,
					DDDIN_RegWrite		=> DD_RegWrite,
					DDDOUT_MemtoReg 	=> DDD_MemtoReg,
					DDDOUT_RegWrite		=> DDD_RegWrite,
					DDDIN_write_register_address	=> DD_write_register_address,
					DDDOUT_write_register_address	=> DDD_write_register_address,
                	clock 				=> clock,
					reset        		=> reset );

END structure;

